
/* 
Include Files
*/
#include <Blackfin_definitions.h>
#include <functions.h>

/*
Global Variables
*/

char CommandBuffer[6][8];
char ResponseBuffer[17][8];
char ResponseTempBuffer[20][8];
char FinalResponseBuffer[17];
char TransmitData[516][8];
char ReceiveData[950][8];
char MultiReceiveData1[514][8];
char MultiReceiveData2[514][8];
char FinalReceiveData[514];
int InterruptCause;
int TimerInterruptCause;
int ClockDone;
int OneFilled;
int PPIDone;
int ReadyForInput;
char MaxSpeed;
char MaxReadBlkLen;
int CardSize;
char Busy;
char SectorSize;
char MaxWriteBlkLen;
int NewRCA;
int TempPFStatus;
int CheckOver;
char *tempptr1;
volatile int ZeroRxd;

/*
Function Definitions
*/

void PLLSetupBF()
{
    *pSIC_IWR = 0x1;
	*pPLL_CTL = SET_MSEL(VCO / CLKin);
	ssync();
	idle();
	*pPLL_DIV = SET_SSEL(VCO / SystemCLK) | ((VCO_CoreCLK)&0x30);
	asm("ssync;");
}

void PortMuxingBF()
{
	*pPORTF_FER = PF0 | PF1 | PF2 | PF3 | PF4;
	*pPORTG_FER = PG10 | PG11 | PG7 | PG8;
	*pPORTG_MUX |= 0x1000;
	*pPORTGIO_DIR = PG12;
	*pPORTGIO_CLEAR = PG12;
}

void SetUpInterruptsBF()
{
	*pSIC_IAR5 = 0x06666655;
    register_handler(ik_ivg8,MMCIsr);
    register_handler(ik_ivg12,PFIsr);
    register_handler(ik_ivg13,CDIsr);
    *pSIC_IMASK0 |= IRQ_DMA0;
}

void MMCInit()
{
	char tempvar;
    int i = 0;
    
    *pTIMER_STATUS = TIMIL4 | TIMIL6;
    
    SendCommand(0,0);        
   do
    {
    	SendCommand(55,0);  	/*Tx - Send CMD 55 - Expect R1 response */
        ReceiveResponse(48);  	/*Rx - Receive R1 response */

        SendCommand(41, 0x00FF8000);
	    ReceiveResponse(48);  	/*Rx - Receive R3 response */
	    ExtractResponse(48);

		//This delay is given to meet the SD Card Spec
		SetUpClockSD(1000);   

		tempvar = FinalResponseBuffer[1] & 0x80;
    } while ((tempvar & 0x80) != 0x80);
    
    SendCommand(2,0);  /*Tx - Send CMD 2 - Expect R2 response */
    ReceiveResponse(136);  /*Rx - Receive R2 response */
    
    SendCommand(3,0);  /*Tx - Send CMD 3 - Expect R6 response */
    ReceiveResponse(48);  /*Rx - Receive R6 response */
    ExtractResponse(48);
   
    NewRCA = (((FinalResponseBuffer[1]<<8) & 0xFF00) | ((FinalResponseBuffer[2]) & 0xFF));
    
	SendCommand(9,((NewRCA<<16) & 0xFFFF0000));  /*Tx - Send CMD 9 - Expect R2 response */
    ReceiveResponse(136);  /*Rx - Receive R2 response */
    ExtractResponse(48);
   
    MaxSpeed = FinalResponseBuffer[4];
    MaxReadBlkLen = (FinalResponseBuffer[6] & 0x0F);
    CardSize = (((FinalResponseBuffer[7] & 0x3)<<10) | (FinalResponseBuffer[8] << 2) | (FinalResponseBuffer[9] & 0xC0));
    SectorSize = (((FinalResponseBuffer[11] & 0x3F)<<1) | ((FinalResponseBuffer[12] & 0x80)>>7));
    MaxWriteBlkLen = (((FinalResponseBuffer[13] & 0x3)<<2) | ((FinalResponseBuffer[14] & 0xC0)>>6));  
    
    SendCommand(7,((NewRCA<<16) & 0xFFFF0000));  /*Tx - Send CMD 7 - Expect R1b response */
    ReceiveResponse(48);  /*Rx - Receive R1b response */
    
    SendCommand(16,0);  //Tx - Send CMD 16 - Expect R1 response 
    ReceiveResponse(48);  //Rx - Receive R1 response 
}

void SingleBlockWrite()
{
    int i;
    //StartCyclesCounter();  
    SetUpTransmitData();
	//StopCyclesCounter();
    
    SendCommand(24,0);  //Tx - Send CMD 24 - Expect R1 response
    ReceiveResponse(48);  //Rx - Receive R1 response 
   
    SetUpClockSD(12);
    
    SetUpDMADataTransmit(4121);
    SetUpAndEnablePPITransmit(4120);
    SetUpClockBFReceive(8);    
    SetUpClockBF(4); //Few clock cycles given only to BF to account for the delay
      
    SetUpClock(4121);
    while(PPIDone);
    DisablePPITransmit(); 
    CheckOver= 1;
        
	while(CheckOver)
	{
		PPIDone=1;
	    InterruptCause = 3; //Actual DMA to receive the response
	    SetUpDMADataReceive(16);
	    SetUpAndEnablePPIReceive();
	    SetUpClockBFReceive(8);
	    SetUpClock(16);
	    while(PPIDone);
	    ExtractData();
	}
   
	SetUpClockSD(12);
   
	CheckOver=1;
	while(CheckOver)
	{
		PPIDone=1;
	    InterruptCause = 3; //Actual DMA to receive the response
	    SetUpDMADataReceive(64);
	    SetUpAndEnablePPIReceive();
	    SetUpClockBFReceive(8);
	    SetUpClock(64);
	    while(PPIDone);
	    ExtractData2();
	}
}

void SingleBlockRead()
{   
	int temp;
//	*pTIMER_DISABLE = TIMDIS7 | TIMDIS4 | TIMDIS3;
 
	
    
    SendCommand(17,0);  /*Tx - Send CMD 17 - Expect R1 response */
    
    /*Dummy PPI Rx DMA to tristate D0 line */
    InterruptCause = 3; //Actual DMA to receive the response
    SetUpDMADataReceive(8);
    SetUpAndEnablePPIReceive();
    SetUpClockBFReceive(24);
    while(PPIDone);
    
    ZeroRxd = 1;
    *pPORTG_FER &= 0xFEFF;
    *pPORTFIO_SET = PF11;
    *pIMASK &= 0xEFFF;
    *pILAT = EVT_IVG12;
    *pIMASK |= 0x1000;
    *pPORTGIO_INEN = PG8;
    
    *pTIMER_DISABLE = TIMDIS7 | TIMDIS4| TIMDIS6;
	while(*pTIMER_STATUS & (TRUN7|TRUN4| TRUN6));
	
	*pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
    *pTIMER_ENABLE = TIMEN7;
	
    while(ZeroRxd);
    
     *pTIMER_DISABLE = TIMDIS7;
	while(*pTIMER_STATUS & TRUN7);
    
     *pPORTFIO_CLEAR = PF11;
     *pPORTG_FER |= PG8;
    *pPORTGIO_INEN = 0;
    /********************
    *pPORTFIO_DIR = 0;
    *pPORTFIO_POLAR = PF9 | PF8;
    *pPORTFIO_EDGE = PF8 | PF9;
    *pPORTFIO_MASKA_SET = PF8;
    *pPORTFIO_MASKB_SET =  PF9;
    asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");
    asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");
    asm("nop;");asm("nop;");asm("nop;");asm("nop;");  
		
    *pILAT = EVT_IVG12;
    *pSIC_IMASK1 |= IRQ_PFA_PORTF; //Use this carefully!
    *pPORTFIO_INEN = PF8;
    **********************/
    

    
    
    
    

    PPIDone=1;
    InterruptCause = 3; //Actual DMA to receive the response
    
    SetUpDMADataReceive(4121);
    SetUpAndEnablePPIReceive();
    SetUpClockBFReceive(2);
    SetUpClock(4121);
    while(PPIDone);
    ExtractRData();
}



void MultipleBlockRead()
{
	SendCommand(18,0);  /*Tx - Send CMD 17 - Expect R1 response */
	PPIDone=1;
	OneFilled=1;
    InterruptCause = 5; //Actual DMA to receive the response
    SetUpDMAMultiReceive(4096);
    SetUpAndEnablePPIReceive();
    SetUpClockBFReceive(8);
    SetUpClock(4096); 
    while(OneFilled);
    ExtractMultiData();
    while(PPIDone);
    //ExtractRData();
}

void SetUpDMAMultiReceive(int NumberOfWords)
{
	*pDMA0_START_ADDR = MultiReceiveData1[0];
    *pDMA0_X_MODIFY = 1;
    *pDMA0_X_COUNT = NumberOfWords;
    *pDMA0_CONFIG = FLOW_STOP | WNR | DMAEN | DI_EN;
}

void ExtractMultiData()
{
	asm("nop;");
}

void SendCommand(int Command, int value)
{
	SetUpClockSD(12);
    SetUpCommandArray(Command, value);
    SetUpDMATransmit(48);
    SetUpAndEnablePPITransmit(47);
    SetUpClockBFReceive(8);
    
    SetUpClockBF(4); //Few clock cycles given only to BF to account for the delay
        
    SetUpClock(48);
    while(PPIDone);
    DisablePPITransmit();     
}

void ReceiveResponse(int Type)
{    	
	PPIDone=1;
    InterruptCause = 3; //Actual DMA to receive the response
    SetUpDMAReceive(Type+24);
    SetUpAndEnablePPIReceive();
    SetUpClockBFReceive(8);
    SetUpClock(Type+24);
    while(PPIDone);
}


void SetUpClockSD(int NumberOfCycles)
{
    *pTIMER_DISABLE = TIMDIS7 | TIMDIS4 | TIMDIS6;
	while(*pTIMER_STATUS & (TRUN7|TRUN4| TRUN6));
		
    *pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
	
	*pTIMER4_WIDTH = (NumberOfCycles * (SystemCLK/TimerSlow));
	*pTIMER4_CONFIG = PWM_OUT | IRQ_ENA | PULSE_HI;

	*pTIMER_ENABLE = TIMEN7 | TIMEN4;
	
	while(!(*pTIMER_STATUS & TIMIL4));
   	*pTIMER_STATUS = TIMIL4;
}

void SetUpClockSDContinuous()
{
    *pTIMER_DISABLE = TIMDIS7 | TIMDIS4| TIMDIS6;
	while(*pTIMER_STATUS & (TRUN7|TRUN4| TRUN6));
	
	*pPORTG_FER &= 0xFEFF;//PG8 is now a PF, not TMR4
	*pPORTGIO_DIR |= PG8; //PG8 direction output
	*pPORTGIO_CLEAR = PG8; //PG8 driven low
	
    *pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
	
	*pTIMER_ENABLE = TIMEN7;
}

void SetUpClockSDPulse()
{
	int ja;
	
	
	*pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
	
	*pTIMER_ENABLE = TIMEN7;
	*pTIMER_DISABLE = TIMDIS7;
	while(*pTIMER_STATUS & (TRUN7));
   /*	for(ja=0; ja<1000;ja++)
   	{
   		asm("nop;");
   	}*/
}

void SetUpClockBFContinuous()
{
	*pTIMER_DISABLE = TIMDIS7 | TIMDIS4 | TIMDIS6;
	while(*pTIMER_STATUS & (TRUN7|TRUN4| TRUN6));
	
	*pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
	
	*pTIMER_ENABLE = TIMEN7;

	*pPORTG_FER &= 0xFBFF;//PG8 is now a PF, not TMR4
	*pPORTGIO_DIR |= PG10; //PG8 direction output
	*pPORTGIO_CLEAR = PG10; //PG8 driven low

}

void SetUpClockBF(int NumberOfCycles)
{
	*pTIMER_DISABLE = TIMDIS7 | TIMDIS4 | TIMDIS6;
	while(*pTIMER_STATUS & (TRUN7|TRUN4| TRUN6));
	
	*pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
	
	*pTIMER0_WIDTH = 2;
    *pTIMER0_CONFIG = PWM_OUT | TIN_SEL | CLK_SEL;
    
    
    *pTIMER6_WIDTH = (NumberOfCycles * (SystemCLK/TimerSlow));
	*pTIMER6_CONFIG = PWM_OUT | IRQ_ENA | PULSE_HI;
    
    *pTIMER_ENABLE = TIMEN7 | TIMEN0 | TIMEN6;
    *pTIMER_DISABLE = TIMDIS0;
 	
 	while(!(*pTIMER_STATUS & TIMIL6));
   	*pTIMER_STATUS = TIMIL6;
}

void SetUpClockBFReceive(int NumberOfCycles)
{
	*pTIMER_DISABLE = TIMDIS7 | TIMDIS4 | TIMDIS6;
	while(*pTIMER_STATUS & (TRUN7|TRUN4| TRUN6));
	
	*pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
	
	*pTIMER6_WIDTH = (NumberOfCycles * (SystemCLK/TimerSlow));
	*pTIMER6_CONFIG = PWM_OUT | IRQ_ENA | PULSE_HI;
    
    *pTIMER_ENABLE = TIMEN7 | TIMEN6;
 	
 	while(!(*pTIMER_STATUS & TIMIL6));
   	*pTIMER_STATUS = TIMIL6;
}

void SetUpClock(int NumberOfCycles)
{
    
   *pTIMER_DISABLE = TIMDIS7 | TIMDIS4 |TIMDIS6;
	while(*pTIMER_STATUS & (TRUN7|TRUN4| TRUN6));
    
	*pTIMER7_WIDTH = SystemCLK / TimerSlow / 2;
	*pTIMER7_PERIOD = SystemCLK / TimerSlow;
	*pTIMER7_CONFIG = PERIOD_CNT | PWM_OUT;
	
	*pTIMER6_WIDTH = (NumberOfCycles * (SystemCLK/TimerSlow));
	*pTIMER6_CONFIG = PWM_OUT | IRQ_ENA | PULSE_HI;
	
	*pTIMER4_WIDTH = (NumberOfCycles * (SystemCLK/TimerSlow));
	*pTIMER4_CONFIG = PWM_OUT | IRQ_ENA | PULSE_HI;

	*pTIMER_ENABLE = TIMEN7 | TIMEN6 | TIMEN4;
	
	while(!(*pTIMER_STATUS & TIMIL4));
   	*pTIMER_STATUS = TIMIL4;
   	
   	while(!(*pTIMER_STATUS & TIMIL6));
   	*pTIMER_STATUS = TIMIL6;
}

void SetUpClockRx(int NumberOfCycles)
{
	
}

void SetUpDMATransmit(int NumberOfWords)
{
    *pDMA0_START_ADDR = CommandBuffer[0];
    *pDMA0_X_COUNT = NumberOfWords;
    *pDMA0_X_MODIFY = 1;
    *pDMA0_CONFIG = FLOW_STOP | DMAEN | DI_EN; 
}

void SetUpDMADataTransmit(int NumberOfWords)
{
    *pDMA0_START_ADDR = TransmitData[0];
    *pDMA0_X_MODIFY = 1;
    *pDMA0_X_COUNT = NumberOfWords;
    *pDMA0_CONFIG = FLOW_STOP | DMAEN | DI_EN; 
}


void SetUpDMAReceive(int NumberOfWords)
{
    *pDMA0_START_ADDR = ResponseTempBuffer[0];
    *pDMA0_X_MODIFY = 1;
    *pDMA0_X_COUNT = NumberOfWords;
    *pDMA0_CONFIG = FLOW_STOP | WNR | DMAEN | DI_EN; 
}

void SetUpDMADataReceive(int NumberOfWords)
{
    *pDMA0_START_ADDR = ReceiveData[0];
    *pDMA0_X_MODIFY = 1;
    *pDMA0_X_COUNT = NumberOfWords;
    *pDMA0_CONFIG = FLOW_STOP | WNR | DMAEN | DI_EN; 
}

void SetUpAndEnablePPITransmit(int NumberOfWords)
{
    InterruptCause = 1;
    PPIDone = 1;
    *pPPI_DELAY = 0;
    *pPPI_COUNT = NumberOfWords;
    *pPPI_CONTROL = PORT_DIR | PORT_EN | POLS | XFR_TYPE;
}

void SetUpAndEnablePPIReceive()
{
    
    PPIDone = 1;
    *pPPI_CONTROL = XFR_TYPE | PORT_CFG | FLD_SEL | PORT_EN | POLS;
}

void SetUpCommandArray(int Command, int value)
{	
    int temp, temp1;
    char crc;
    int i,j,a,k,xor;
    char Data;
    
    for(i=0;i<6;i++)
	{
	   for(j=0;j<8;j++)
	   {
	       CommandBuffer[i][j] = 0;
	   }
	}
    // Filling up the command buffer
    CommandBuffer[0][0] = 0x40 | Command;
    CommandBuffer[1][0] = (value & 0xff000000) >> 24;
	CommandBuffer[2][0] = (value & 0xff0000) >> 16;
	CommandBuffer[3][0] = (value & 0xff00) >> 8;
	CommandBuffer[4][0] = value & 0xff;
	
	// CMD0 should be 0x40 00 00 00 00 95

	    crc=0; 
		for (a=0;a<5;a++) 
		{ 
			Data=CommandBuffer[a][0]; 
			for (k=0;k<8;k++) 
			{ 
				crc <<= 1; 

				if ((Data & 0x80)^(crc & 0x80)) 
				crc ^=0x09; 
				Data <<= 1; 
			} 
		} 
		crc=(crc<<1)|1; 
		CommandBuffer[5][0] = crc;
		
			
	for(i=0;i<6;i++)
	{
	   temp = CommandBuffer[i][0]; 
	   for(j=0;j<8;j++)
	   {
	       temp1 = (temp & (1<<j));
	       if(j<4)
	       {
	           CommandBuffer[i][7-j] = 0xF | (temp1 << (4-j));
	       }
	       if(j==4)
	       {
	           CommandBuffer[i][7-j] = 0xF | temp1; 
	       }
	       if(j>4)
	       {
	           CommandBuffer[i][7-j] = 0xF | (temp1 >> (j-4));
	       }
	   }
	}
}

void SetUpTransmitData()
{
	
	short crc, Data, temp;
	char DataIn;
	short a, b, c;
	int i, TempData;
	char j;
	
	TransmitData[0][0] = 0xFE;
        
    crc=0; 
    i=0;
	for (i=1;i<513;i++) 
	{
		for(j=0;j<8;j++)
		{
			DataIn = (TransmitData[i][0] & (1<<(7-j))) >> (7-j);
			a = (crc & 0x1)	^ DataIn;
			b = ((crc & 0x800)>>11) ^ a;
			c = ((crc & 0x10)>>4) ^ a;
			crc = crc >> 1;
			crc = crc & 0x7BF7;
			a = a << 15;
			b = b << 10;
			c = c << 3;
			crc = crc | (a & 0x8000);
			crc = crc | (b & 0x400);
			crc = crc | (c & 0x8);
		}
	}
	temp = 0;
	for(j=0;j<16;j++)
	{
		temp = temp | (((crc & (1<<j))>>j)<<(15-j));
	}
	
	TransmitData[513][0] = ((temp&0xFF00)>>8);
    TransmitData[514][0] = (temp&0xFF);
	
	for(i=0;i<515;i++)
	{
	   TempData = TransmitData[i][0]; 
	   for(j=0;j<8;j++)
	   {
	       TransmitData[i][j] = (0x1E | ((TempData & (1<<(7-j)))>>(7-j)));
	   }
	}
	TransmitData[515][0] = 0x1F;
}

void ExtractResponse(Type)
{
    int i,j, tt, ja, ia, k, l;
    char *tempptr;
    
    for(i=0;i<20;i++)
    {
    	for(j=0;j<8;j++)
    	{
    		if((ResponseTempBuffer[i][j] & 0x10) == 0)
    		{
    			goto labj;
    		}
    	}
    }
    
labj: if((i>=20) & (j>=8))
    {
    	while(1);
    }
    
    else
    {
    	tt = (20*8)-((i-1)*j) + j;
    	if(tt<Type)
    	{
    		while(1);
    	}
    
    	else
    	{
    		tempptr = &ResponseTempBuffer[i][j];
    		for(i=0;i<17;i++)
    		{
    			for(j=0;j<8;j++)
    			{
    				ResponseBuffer[i][j] = *tempptr++;
    			}
    		}
    	}
    }
    		/*tempptr = &ResponseTempBuffer[i][j];
    		for(i=0;i<17;i++)
    		{
    			for(j=0;j<8;j++)
    			{
    				ResponseBuffer[i][j] = *tempptr++;
    				if(tempptr >= &ResponseTempBuffer[i][j] + */
    		/*for(ja=j,l=0 ; ja<8; ja++, l++)
    		{
    			ResponseBuffer[0][l] = ResponseTempBuffer[i][ja];
    		}
    		for(ia=i+1; ia<19; ia++)
    		{
    			for(ja=0; ja<8; ja++)
    			{
    				if(l==8)
    				{
    					l=0;
    				}
    				ResponseBuffer[ia-i][l] = ResponseTempBuffer[i][ja];
    			}
    		}*/
    	


	
    for(i=0;i<17;i++)
    {
        FinalResponseBuffer[i] = 0;
    }
    for(i=0;i<17;i++)
    {
     	for(j=0;j<8;j++)
     	{
     	    ResponseBuffer[i][j] = ResponseBuffer[i][j] & (0x10);
     	    if(j<3)
     	    {
     	        ResponseBuffer[i][j] = ResponseBuffer[i][j] << (3-j);
     	    }
     	    if(j>3)
     	    {
     	        ResponseBuffer[i][j] = ResponseBuffer[i][j] >> (j-3);   	        
     	    }
     	    FinalResponseBuffer[i] |= ResponseBuffer[i][j];
     	}   
     }    
}

void ExtractData()
{
	int i,j;
	
	for(i=0;i<8;i++)
    {
    	for(j=0;j<8;j++)
    	{
    		if((ReceiveData[i][j] & 0x1) == 0)
    		{
    			CheckOver = 0;
    		}
    	}
    }

}

void ExtractData2()
{
	int i,j;
	
	for(i=0;i<8;i++)
    {
    	for(j=0;j<8;j++)
    	{
    		if((ReceiveData[i][j] & 0x1) == 1)
    		{
    			CheckOver = 0;
    		}
    	}
    }

}



void ExtractRData()
{
	int i,j, tt, ja, ia, k, l;
    
/*	for(i=0;i<514;i++)
    {
        FinalReceiveData[i] = 0;
    }
	
    
    for(i=0;i<950;i++)
    {
    	for(j=0;j<8;j++)
    	{
    		if((ReceiveData[i][j] & 0x01) == 0)
    		{
    			goto labj;
    		}
    	}
    }
    
labj: 
    
	tempptr1 = &ReceiveData[i][j];
	tempptr1++;
	
	for(i=0;i<514;i++)
	{
		for(j=0;j<8;j++)
		{
			ReceiveData[i][j] = *tempptr1++;
		}
	}*/
    
        for(i=0;i<514;i++)
    {
     	for(j=0;j<8;j++)
     	{
     	    ReceiveData[i][j] = ReceiveData[i][j] & (0x1);
     	    ReceiveData[i][j] = ReceiveData[i][j] << (7-j);
     	    
     	    FinalReceiveData[i] |= ReceiveData[i][j];
     	}   
     }    
     
   
}

void SetupPFpin()
{
    *pPORTFIO_DIR |= PF11;
    *pPORTFIO_CLEAR = PF11;
    
    *pPORTGIO_DIR = PG12;
    *pPORTGIO_POLAR = PG8;
    *pPORTGIO_EDGE = PG8;
    *pPORTGIO_MASKA_SET = PG8;
    *pILAT = EVT_IVG12;
     *pSIC_IMASK1 |= IRQ_PFA_PORTG;//Use this carefully!
    
}

void CardDetection()
{
	*pPORTFIO_DIR = PF13;
	*pPORTFIO_SET = PF13;
    *pPORTFIO_POLAR = 0;
    *pPORTFIO_EDGE = 0;
    *pPORTFIO_MASKB_SET = PF12;
    *pSIC_IMASK1 |= IRQ_PFB_PORTF; //Use this carefully!
    *pPORTFIO_INEN = PF12;
}

void SetupPFInterrupt()
{
	*pPORTFIO_DIR = 0;
    *pPORTFIO_POLAR = PF9 | PF8;
    *pPORTFIO_EDGE = PF8 | PF9;
    *pPORTFIO_MASKA_SET = PF8;
    *pPORTFIO_MASKB_SET =  PF9;
    asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");
    asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");asm("nop;");
    asm("nop;");asm("nop;");asm("nop;");asm("nop;");  
	
	
    *pILAT = EVT_IVG12;
    *pSIC_IMASK1 |= IRQ_PFA_PORTF; //Use this carefully!
    *pPORTFIO_INEN = PF8;
}

void DisablePPITransmit()
{
   while(*pDMA0_IRQ_STATUS & DMA_RUN);
    *pPPI_CONTROL = 0;
    *pDMA0_CONFIG = 0x0;
}

void DisablePPIReceive()
{
    *pDMA0_CONFIG = 0x0;
    *pPPI_CONTROL = 0;
}

void StartCyclesCounter()
{
	asm("R2 = 0;");
	asm("CYCLES = R2;");
	asm("CYCLES2 = R2;");
	asm("R2 = SYSCFG;");
	asm("BITSET(R2, 1);");
	asm("SYSCFG = R2;");	
}

void StopCyclesCounter()
{
	asm("R2 = SYSCFG;");
	asm("BITCLR(R2, 1);");
	asm("SYSCFG = R2;");
}
